@@ -40,10 +40,6 @@ const no_space_table_t add_space_table[] =
    { CT_ATTRIBUTE,             CT_TYPE                  },
    { CT_ATTRIBUTE,             CT_WORD                  },
    { CT_AUTORELEASEPOOL,       CT_BRACE_OPEN            },
-   { CT_BIT_COLON,             CT_NUMBER                },
-   { CT_BIT_COLON,             CT_SIZEOF                },
-   { CT_BIT_COLON,             CT_TYPE                  },
-   { CT_BIT_COLON,             CT_WORD                  },
    { CT_BODY,                  CT_BRACE_OPEN            },
    { CT_BRACE_CLOSE,           CT_BRACE_OPEN            },
    { CT_BRACE_CLOSE,           CT_CLASS                 },
@@ -100,7 +96,6 @@ const no_space_table_t add_space_table[] =
    { CT_ENUM_CLASS,            CT_ATTRIBUTE             },
    { CT_ENUM_CLASS,            CT_TYPE                  },
    { CT_ENUM,                  CT_ATTRIBUTE             },
-   { CT_ENUM,                  CT_BIT_COLON             },
    { CT_ENUM,                  CT_BRACE_OPEN            },
    { CT_ENUM,                  CT_ENUM_CLASS            },
    { CT_ENUM,                  CT_FPAREN_OPEN           },
@@ -251,14 +246,12 @@ const no_space_table_t add_space_table[] =
    { CT_STRING,                CT_CONCAT                },
    { CT_STRING,                CT_OC_MSG_NAME           },
    { CT_STRING,                CT_PAREN_OPEN            },
-   { CT_STRING,                CT_STRING                },
    { CT_STRING,                CT_STRUCT                },
    { CT_STRING,                CT_TYPE                  },
    { CT_STRING,                CT_WORD                  },
    { CT_STRUCT,                CT_BRACE_OPEN            },
    { CT_STRUCT,                CT_CLASS_COLON           },
    { CT_STRUCT,                CT_MACRO_FUNC_CALL       },
-   { CT_STRUCT,                CT_TYPE                  },
    { CT_STRUCT,                CT_WORD                  },
    { CT_TEMPLATE,              CT_CLASS                 },
    { CT_TEMPLATE,              CT_TYPE                  },
@@ -291,7 +284,6 @@ const no_space_table_t add_space_table[] =
    { CT_WHERE_COLON,           CT_WORD                  },
    { CT_WHERE_SPEC,            CT_WORD                  },
    { CT_WORD,                  CT_ATTRIBUTE             },
-   { CT_WORD,                  CT_BIT_COLON             },
    { CT_WORD,                  CT_BRACE_OPEN            },
    { CT_WORD,                  CT_CLASS_COLON           },
    { CT_WORD,                  CT_COLON                 },
@@ -363,4 +355,18 @@ const no_space_table_t no_space_table[] =
    { CT_TYPENAME,       CT_TYPE         },
 };
 
+/**
+ * this table lists out all combos where nothing is to do
+ * CT_UNKNOWN is a wildcard.
+ *
+ * TODO: some of these are no longer needed.
+ */
+const no_space_table_t IGNORE_space_table[] =
+{
+   { CT_NUMBER_FP,    CT_PAREN_OPEN},
+   { CT_PAREN_CLOSE,    CT_SIZEOF},
+   { CT_SQUARE_CLOSE,   CT_INCDEC_BEFORE},
+};
+
+
 // *INDENT-ON*
