@@ -1,6 +1,6 @@
 /**
  * @file indent.h
- * prototypes for indent.c
+ * prototypes for indent.cpp
  *
  * @author  Ben Gardner
  * @license GPL v2+
@@ -43,17 +43,4 @@ void indent_to_column(Chunk *pc, size_t column);
 void align_to_column(Chunk *pc, size_t column);
 
 
-//! Scan to see if the whole file is covered by one #ifdef
-bool ifdef_over_whole_file();
-
-
-/**
- * Changes the initial indent for a line to the given column
- *
- * @param pc      The chunk at the start of the line
- * @param column  The desired column
- */
-void reindent_line(Chunk *pc, size_t column);
-
-
 #endif /* INDENT_H_INCLUDED */
