@@ -7,10 +7,10 @@
  * @author  Matthew Woehlke since version 0.67
  * @license GPL v2+
  */
-
 #include "option.h"
 
 #include "keywords.h"
+#include "language_names.h"
 #include "uncrustify.h"
 #include "uncrustify_version.h"
 
@@ -27,7 +27,7 @@ namespace uncrustify
 namespace
 {
 
-static const char *DOC_TEXT_END = u8R"___(
+static const char *DOC_TEXT_END = R"___(
 # Meaning of the settings:
 #   Ignore - do not do any changes
 #   Add    - makes sure there is 1 or more space/brace/newline/etc
@@ -308,6 +308,16 @@ void print_description(FILE *pfile, std::string description,
 }
 
 
+//-----------------------------------------------------------------------------
+void deprecated_stop_or_not()
+{
+   if (cpd.find_deprecated)
+   {
+      exit(EX_OK);
+   }
+}
+
+
 //-----------------------------------------------------------------------------
 bool process_option_line_compat_0_68(const std::string              &cmd,
                                      const std::vector<std::string> &args,
@@ -316,9 +326,8 @@ bool process_option_line_compat_0_68(const std::string              &cmd,
    if (cmd == "sp_cpp_lambda_paren")
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
-      w("option '%s' is deprecated; use '%s' instead",
+      w("option '%s' is deprecated; use '%s' instead.\n",
         cmd.c_str(), options::sp_cpp_lambda_square_paren.name());
-
       UNUSED(options::sp_cpp_lambda_square_paren.read(args[1].c_str()));
       return(true);
    }
@@ -331,10 +340,9 @@ bool process_option_line_compat_0_70(const std::string &cmd, const char *filenam
    if (cmd == "sp_word_brace")                     // Issue #2428
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
-      w("option '%s' is deprecated; did you want to use '%s' instead?",
+      w("option '%s' is deprecated; did you want to use '%s' instead?\n",
         cmd.c_str(), options::sp_type_brace_init_lst.name());
-
-      //UNUSED(options::sp_type_brace_init_lst.read(args[1].c_str()));
+      deprecated_stop_or_not();
       return(true);
    }
    return(false);
@@ -346,30 +354,27 @@ bool process_option_line_compat_0_73(const std::string &cmd, const char *filenam
    if (cmd == "indent_sing_line_comments")         // Issue #3249
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
-      w("option '%s' is deprecated; did you want to use '%s' instead?",
+      w("option '%s' is deprecated; did you want to use '%s' instead?\n",
         cmd.c_str(), options::indent_single_line_comments_before.name());
-
-      //UNUSED(options::indent_single_line_comments_before.read(args[1].c_str()));
+      deprecated_stop_or_not();
       return(true);
    }
 
    if (cmd == "sp_before_tr_emb_cmt")              // Issue #3339
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
-      w("option '%s' is deprecated; did you want to use '%s' instead?",
+      w("option '%s' is deprecated; did you want to use '%s' instead?\n",
         cmd.c_str(), options::sp_before_tr_cmt.name());
-
-      //UNUSED(options::sp_before_tr_cmt.read(args[1].c_str()));
+      deprecated_stop_or_not();
       return(true);
    }
 
    if (cmd == "sp_num_before_tr_emb_cmt")          // Issue #3339
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
-      w("option '%s' is deprecated; did you want to use '%s' instead?",
+      w("option '%s' is deprecated; did you want to use '%s' instead?\n",
         cmd.c_str(), options::sp_num_before_tr_cmt.name());
-
-      //UNUSED(options::sp_num_before_tr_cmt.read(args[1].c_str()));
+      deprecated_stop_or_not();
       return(true);
    }
    return(false);
@@ -381,9 +386,9 @@ bool process_option_line_compat_0_74(const std::string &cmd, const char *filenam
    if (cmd == "sp_type_question")         // PR #3638
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
-      w("option '%s' is deprecated; did you want to use '%s' instead?",
+      w("option '%s' is deprecated; did you want to use '%s' instead?\n",
         cmd.c_str(), options::sp_before_ptr_star.name());
-
+      deprecated_stop_or_not();
       return(true);
    }
    return(false);
@@ -395,9 +400,9 @@ bool process_option_line_compat_0_75(const std::string &cmd, const char *filenam
    if (cmd == "pp_space")
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
-      w("option '%s' is deprecated; it has been replaced by '%s'.",
+      w("option '%s' is deprecated; it has been replaced by '%s'.\n",
         cmd.c_str(), options::pp_space_after.name());
-
+      deprecated_stop_or_not();
       return(true);
    }
 
@@ -405,7 +410,7 @@ bool process_option_line_compat_0_75(const std::string &cmd, const char *filenam
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
       w("option '%s' is deprecated; it was a temporary option used\n"
-        "during the development of version 0.76. Use '%s' and '%s' instead.",
+        "during the development of version 0.76. Use '%s' and '%s' instead.\n",
         cmd.c_str(), options::pp_indent.name(), options::pp_indent_count.name());
 
       return(true);
@@ -420,16 +425,39 @@ bool process_option_line_compat_0_76(const std::string &cmd, const std::vector<s
    {
       OptionWarning w{ filename, OptionWarning::MINOR };
       w("option '%s' is deprecated; it has been replaced by '%s'.\n"
-        "You can also use '%s' for additional functionality",
+        "You can also use '%s' for additional functionality.\n",
         cmd.c_str(), options::nl_var_def_blk_end_func_top.name(),
         options::nl_var_def_blk_end.name());
-
+      deprecated_stop_or_not();
       UNUSED(options::nl_var_def_blk_end_func_top.read(args[1].c_str()));
       return(true);
    }
    return(false);
 } // process_option_line_compat_0_76
 
+
+bool process_option_line_compat_0_78(const std::string &cmd, const char *filename)
+{
+   if (cmd == "pp_warn_unbalanced_if")
+   {
+      OptionWarning w{ filename, OptionWarning::MINOR };
+      w("option '%s' is deprecated; it has been replaced by '%s'.\n",
+        cmd.c_str(), options::pp_unbalanced_if_action.name());
+      deprecated_stop_or_not();
+      return(true);
+   }
+
+   if (cmd == "sp_balance_nested_parens")
+   {
+      OptionWarning w{ filename, OptionWarning::MINOR };
+      w("option '%s' never works; it has been replaced by '%s'.\n",
+        cmd.c_str(), options::sp_paren_paren.name());
+      deprecated_stop_or_not();
+      return(true);
+   }
+   return(false);
+} // process_option_line_compat_0_78
+
 } // namespace
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1033,7 +1061,7 @@ void process_option_line(const std::string &config_line, const char *filename,
       else if (is_path_relative(include_path))
       {
          // include is a relative path to the current config file
-         unc_text ut = std::string{ filename };
+         UncText ut = std::string{ filename };
          ut.resize(static_cast<unsigned>(path_dirname_len(filename)));
          ut.append(include_path);
          UNUSED(load_option_file(ut.c_str(), compat_level));
@@ -1138,6 +1166,14 @@ void process_option_line(const std::string &config_line, const char *filename,
             return;
          }
       }
+
+      if (compat_level < option_level(0, 79))
+      {
+         if (process_option_line_compat_0_78(cmd, filename))
+         {
+            return;
+         }
+      }
       const auto oi = option_map.find(cmd);
 
       if (oi == option_map.end())
@@ -1209,6 +1245,12 @@ bool load_option_file(const char *filename, int compat_level)
       ++cpd.line_number;
       process_option_line(line, filename, compat_level);
    }
+
+   if (cpd.find_deprecated)
+   {
+      fprintf(stderr, "no deprecated option found.\n");
+      exit(EX_OK);
+   }
    return(true);
 } // load_option_file
 
